from .karate import KarateClub
from .tu_dataset import TUDataset
from .planetoid import Planetoid
from .ppi import PPI
from .qm7 import QM7b
from .qm9 import QM9
from .entities import Entities
from .mnist_superpixels import MNISTSuperpixels
from .faust import FAUST
from .shapenet import ShapeNet
from .modelnet import ModelNet
from .coma import CoMA

__all__ = [
    'KarateClub',
    'TUDataset',
    'Planetoid',
    'PPI',
    'QM7b',
    'QM9',
    'Entities',
    'MNISTSuperpixels',
    'FAUST',
    'ShapeNet',
    'ModelNet',
    'CoMA',
]
